import React, { useEffect, useState } from "react";
import { HexColorPicker } from "react-colorful";
import { useDispatch } from "react-redux";
import { useParams } from "react-router-dom";

import {
  addNewTagToProject,
  deleteTagProject,
  setProjectBoardFetch
} from "@redux/projectsTrackerSlice";

import { apiRequest } from "@api/request";

import { useNotification } from "@hooks/useNotification";

import arrowDown from "assets/icons/arrows/selectArrow.png";
import close from "assets/icons/close.png";
import edit from "assets/icons/edit.svg";

import "./trackerTagList.scss";

const TrackerTagList = ({ projectBoard }) => {
  const dispatch = useDispatch();
  const projectId = useParams();

  const { showNotification } = useNotification();
  const [tagInfo, setTagInfo] = useState({ description: "", name: "" });
  const [color, setColor] = useState("#aabbcc");
  const [tags, setTags] = useState({
    open: false,
    add: false,
    edit: false
  });

  function deleteTag(tagId) {
    apiRequest("/mark/detach", {
      method: "DELETE",
      data: {
        mark_id: tagId,
        entity_type: 1,
        entity_id: projectId.id
      }
    }).then(() => {
      dispatch(deleteTagProject(tagId));
      showNotification({
        show: true,
        text: "Тег удален",
        type: "success"
      });
    });
  }

  function addNewTag() {
    apiRequest("/mark/create", {
      method: "POST",
      data: {
        title: tagInfo.description,
        slug: tagInfo.name,
        color: color,
        status: 1
      }
    }).then((data) => {
      apiRequest("/mark/attach", {
        method: "POST",
        data: {
          mark_id: data.id,
          entity_type: 1,
          entity_id: projectId.id
        }
      }).then((data) => {
        dispatch(addNewTagToProject(data.mark));
        setTags((prevState) => ({
          ...prevState,
          add: false
        }));
        showNotification({
          show: true,
          text: "Тег успешно создан",
          type: "success"
        });
      });
    });
  }

  function editTag() {
    apiRequest("/mark/update", {
      method: "PUT",
      data: {
        mark_id: tagInfo.editMarkId,
        title: tagInfo.description,
        slug: tagInfo.name,
        color: color
      }
    }).then(() => {
      dispatch(setProjectBoardFetch(projectId.id));
      setTags((prevState) => ({
        ...prevState,
        edit: false
      }));
      setTagInfo({ description: "", name: "" });
      setColor("#aabbcc");
      showNotification({
        show: true,
        text: "Тег успешно изменён",
        type: "success"
      });
    });
  }

  const initListeners = () => {
    document.addEventListener("click", closeByClickingOut);
  };

  const closeByClickingOut = (event) => {
    const path = event.path || (event.composedPath && event.composedPath());
    if (
      event &&
      !path.find(
        (div) =>
          div.classList &&
          (div.classList.contains("tasks__head__tags") ||
            div.classList.contains("tags__list"))
      )
    ) {
      setTags({ open: false, add: false, edit: false });
      setTagInfo({ description: "", name: "" });
      setColor("#aabbcc");
    }
  };

  useEffect(() => {
    initListeners();
  }, []);

  return (
    <div className="tasks__head__tags">
      <div
        className="tags__add"
        onClick={() => {
          setTags((prevState) => ({
            ...prevState,
            open: !tags.open
          }));
        }}
      >
        <p>Список тегов</p>
        <img className={tags.open ? "open" : ""} src={arrowDown} alt="arrow" />
      </div>
      {tags.open && (
        <div className="tags__list">
          {!tags.add && !tags.edit && (
            <div
              className="add-new-tag"
              onClick={() =>
                setTags((prevState) => ({
                  ...prevState,
                  add: true
                }))
              }
            >
              <p>Добавить новый тег</p>
              <span>+</span>
            </div>
          )}
          {!tags.add && !tags.edit && (
            <div className="tags__list__created">
              {projectBoard.mark.map((tag) => {
                return (
                  <div className="tag-item" key={tag.id}>
                    <div className="tag-item__info">
                      <span
                        className="tag-item__color"
                        style={{ background: tag.color }}
                      />
                      <div>
                        <span className="tag-item__info__name">{tag.slug}</span>
                        <p className="tag-item__description">{tag.title}</p>
                      </div>
                    </div>
                    <div className="tag-item__images">
                      <img
                        src={edit}
                        alt="edit"
                        onClick={() => {
                          setTags((prevState) => ({
                            ...prevState,
                            edit: true
                          }));
                          setTagInfo({
                            description: tag.title,
                            name: tag.slug,
                            editMarkId: tag.id
                          });
                          setColor(tag.color);
                        }}
                      />
                      <img
                        onClick={() => deleteTag(tag.id)}
                        className="delete"
                        src={close}
                        alt="delete"
                      />
                    </div>
                  </div>
                );
              })}
            </div>
          )}
          {(tags.add || tags.edit) && (
            <div className="form-tag">
              <input
                className="form-tag__input"
                placeholder="Описание метки"
                maxLength="25"
                value={tagInfo.description}
                onChange={(e) =>
                  setTagInfo((prevState) => ({
                    ...prevState,
                    description: e.target.value
                  }))
                }
              />
              <input
                className="form-tag__input"
                placeholder="Тег"
                value={tagInfo.name}
                maxLength="10"
                onChange={(e) =>
                  setTagInfo((prevState) => ({
                    ...prevState,
                    name: e.target.value
                  }))
                }
              />
              <HexColorPicker color={color} onChange={setColor} />
              <button
                onClick={() => {
                  tags.add ? addNewTag() : editTag();
                }}
                className={
                  tagInfo.name && tagInfo.description
                    ? "form-tag__btn"
                    : "form-tag__btn disable"
                }
              >
                {tags.add ? "Добавить" : "Изменить"}
              </button>
              {(tags.add || tags.edit) && (
                <button
                  className={"form-tag__btn"}
                  onClick={() => {
                    setTags((prevState) => ({
                      ...prevState,
                      add: false,
                      edit: false
                    }));
                    setTagInfo({
                      description: "",
                      name: ""
                    });
                    setColor("#aabbcc");
                  }}
                >
                  Отмена
                </button>
              )}
            </div>
          )}
        </div>
      )}
    </div>
  );
};

export default TrackerTagList;
